﻿using System;
using System.Collections.Generic;
using System.Drawing;
using System.IO;
using System.Linq;
using System.Text;
using C1.FlexMap;
using C1Map_CustomSources;

namespace C1Map.Sources
{
    interface IRequireAttribution
    {
        string Attribution { get; }
    }

    /// <summary>
    /// Custom online Tile Source that gets tile images from OpenStreeMap
    /// </summary>
    public class OpenStreetTileSource : ITileSource, IRequireAttribution
    {
        private const string UrlTemplate = "http://tile.openstreetmap.org/{0}/{1}/{2}.png";

        public void GetTile(int zoomLevel, int tilePositionX, int tilePositionY, out string url, out object image)
        {
            image = null;
            url = string.Format(UrlTemplate, zoomLevel, tilePositionX, tilePositionY);
        }

        public int TileWidth { get { return 256; } }
        public int TileHeight { get { return 256; } }

        public string Attribution { get; } = StringResources.Attribution_OpenStreet;
    }

    /// <summary>
    /// Custom online Tile Source that gets tile images from Stamen Maps with Toner map style
    /// </summary>
    public class StamenTonerTileSource : StamenTileSource
    {
        public StamenTonerTileSource() :base(StamenMapType.Toner,StringResources.Attribution_Toner_Terrain)
        {
            
        }
    }
    /// <summary>
    /// Custom online Tile Source that gets tile images from Stamen Maps with Terrain map style
    /// </summary>
    public class StamenTerrainTileSource : StamenTileSource
    {
        public StamenTerrainTileSource() : base(StamenMapType.Terrain, StringResources.Attribution_Toner_Terrain)
        {

        }
    }
    /// <summary>
    /// Custom online Tile Source that gets tile images from Stamen Maps with Watercolor map style
    /// </summary>
    public class StamenWaterColorTileSource : StamenTileSource
    {
        public StamenWaterColorTileSource() : base(StamenMapType.Watercolor, StringResources.Attibution_WaterColor)
        {

        }
    }

    public class StamenTileSource : ITileSource, IRequireAttribution
    {
        public StamenTileSource(StamenMapType type, string attribution )
        {
            UrlTemplate = $"http://stamen-tiles-a.a.ssl.fastly.net/{type.ToString().ToLower()}/" + "{0}/{1}/{2}.png";
            Attribution = attribution;
        }
        private string UrlTemplate;

        public void GetTile(int level, int x, int y, out string url, out object image)
        {
            image = null;
            url = string.Format(UrlTemplate, level, x, y);
        }

        public int TileWidth { get { return 256; } }
        public int TileHeight { get { return 256; } }
        public string Attribution { get; }
    }

    /// <summary>
    /// Custom offline Tile source that shows images from a local location
    /// </summary>
    public class OfflineTilesSource : ITileSource, IRequireAttribution
    {
        private readonly string tilesLocation;
        public OfflineTilesSource(string tilesLocation)
        {
            if (Directory.Exists(tilesLocation))
            {
                this.tilesLocation = Path.Combine(tilesLocation);
            }
            else
                throw new DirectoryNotFoundException();
        }
        public int TileWidth { get { return 256; } }
        public int TileHeight { get { return 256; } }

        public string Attribution { get; } = StringResources.Attribution_OpenStreet;

        public void GetTile(int level, int x, int y, out string uri, out object image)
        {
            uri = string.Empty;
            image = null;

            var imgPath = Path.Combine(tilesLocation, level.ToString(), x.ToString(), (y.ToString() + ".png"));
            if (File.Exists(imgPath))
            {
                using (Bitmap bmp = new Bitmap(imgPath))
                {
                    image = new Bitmap(bmp);
                }
            }
        }

    }

    public enum StamenMapType { 
        Toner,
        Terrain,
        Watercolor
    }
}
